@@ -1,4 +1,4 @@
-/*	$OpenBSD: extern.h,v 1.65 2021/07/13 18:39:39 job Exp $ */
+/*	$OpenBSD: extern.h,v 1.67 2021/09/09 14:15:49 claudio Exp $ */
 /*
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
  *
@@ -364,8 +364,6 @@ extern int verbose;
 
 /* Routines for RPKI entities. */
 
-int		 base64_decode(const unsigned char *, unsigned char **,
-		    size_t *);
 void		 tal_buffer(struct ibuf *, const struct tal *);
 void		 tal_free(struct tal *);
 struct tal	*tal_parse(const char *, char *);
@@ -412,9 +410,9 @@ int		 valid_uri(const char *, size_t, const char *);
 
 /* Working with CMS. */
 unsigned char	*cms_parse_validate(X509 **, const char *,
-			const char *, size_t *);
+		    const ASN1_OBJECT *, size_t *);
 int		 cms_econtent_version(const char *, const unsigned char **,
-			size_t, long *);
+		    size_t, long *);
 /* Helper for ASN1 parsing */
 int		 ASN1_frame(const char *, size_t,
 			const unsigned char **, long *, int *);
@@ -499,6 +497,7 @@ void		 cryptoerrx(const char *, ...)
 
 int		 base64_decode(const unsigned char *, unsigned char **,
 		    size_t *);
+int		 base64_encode(const unsigned char *, size_t, char **);
 char		*hex_encode(const unsigned char *, size_t);
 
 
