@@ -1,4 +1,4 @@
-/*	$OpenBSD: http.c,v 1.34 2021/07/23 16:03:47 job Exp $  */
+/*	$OpenBSD: http.c,v 1.40 2021/09/23 13:26:51 tb Exp $  */
 /*
  * Copyright (c) 2020 Nils Fisher <nils_fisher@hotmail.com>
  * Copyright (c) 2020 Claudio Jeker <claudio@openbsd.org>
@@ -70,6 +70,7 @@
 #define HTTP_USER_AGENT		"OpenBSD rpki-client"
 #define HTTP_BUF_SIZE		(32 * 1024)
 #define HTTP_IDLE_TIMEOUT	10
+#define HTTP_IO_TIMEOUT		(3 * 60)
 #define MAX_CONNECTIONS		64
 #define NPFDS			(MAX_CONNECTIONS + 1)
 
@@ -83,6 +84,9 @@ enum http_state {
 	STATE_FREE,
 	STATE_CONNECT,
 	STATE_TLSCONNECT,
+	STATE_PROXY_REQUEST,
+	STATE_PROXY_STATUS,
+	STATE_PROXY_RESPONSE,
 	STATE_REQUEST,
 	STATE_RESPONSE_STATUS,
 	STATE_RESPONSE_HEADER,
@@ -96,9 +100,9 @@ enum http_state {
 
 struct http_proxy {
 	char	*proxyhost;
-	char	*proxyuser;
-	char	*proxypw;
-};
+	char	*proxyport;
+	char	*proxyauth;
+} proxy;
 
 struct http_connection {
 	LIST_ENTRY(http_connection)	entry;
@@ -116,6 +120,7 @@ struct http_connection {
 	size_t			bufpos;
 	off_t			iosz;
 	time_t			idle_time;
+	time_t			io_time;
 	int			status;
 	int			fd;
 	int			chunked;
@@ -148,8 +153,10 @@ static size_t http_conn_count;
 static struct msgbuf msgq;
 static struct sockaddr_storage http_bindaddr;
 static struct tls_config *tls_config;
+#ifdef HAVE_PLEDGE
 static uint8_t *tls_ca_mem;
 static size_t tls_ca_size;
+#endif
 
 /* HTTP request API */
 static void	http_req_new(size_t, char *, char *, int);
@@ -177,10 +184,13 @@ static enum res http_handle(struct http_connection *);
 
 /* Internal state functions used by the above functions */
 static enum res	http_finish_connect(struct http_connection *);
+static enum res	proxy_connect(struct http_connection *);
 static enum res	http_tls_connect(struct http_connection *);
 static enum res	http_tls_handshake(struct http_connection *);
 static enum res	http_read(struct http_connection *);
 static enum res	http_write(struct http_connection *);
+static enum res	proxy_read(struct http_connection *);
+static enum res	proxy_write(struct http_connection *);
 static enum res	data_write(struct http_connection *);
 
 static time_t
@@ -277,6 +287,139 @@ url_encode(const char *path)
 	return (epath);
 }
 
+static char
+hextochar(const char *str)
+{
+	unsigned char c, ret;
+
+	c = str[0];
+	ret = c;
+	if (isalpha(c))
+		ret -= isupper(c) ? 'A' - 10 : 'a' - 10;
+	else
+		ret -= '0';
+	ret *= 16;
+
+	c = str[1];
+	ret += c;
+	if (isalpha(c))
+		ret -= isupper(c) ? 'A' - 10 : 'a' - 10;
+	else
+		ret -= '0';
+	return ret;
+}
+
+static char *
+url_decode(const char *str)
+{
+	char *ret, c;
+	int i, reallen;
+
+	if (str == NULL)
+		return NULL;
+	if ((ret = malloc(strlen(str) + 1)) == NULL)
+		err(1, "Can't allocate memory for URL decoding");
+	for (i = 0, reallen = 0; str[i] != '\0'; i++, reallen++, ret++) {
+		c = str[i];
+		if (c == '+') {
+			*ret = ' ';
+			continue;
+		}
+		/*
+		 * Cannot use strtol here because next char
+		 * after %xx may be a digit.
+		 */
+		if (c == '%' && isxdigit((unsigned char)str[i + 1]) &&
+		    isxdigit((unsigned char)str[i + 2])) {
+			*ret = hextochar(&str[i + 1]);
+			i += 2;
+			continue;
+		}
+		*ret = c;
+	}
+	*ret = '\0';
+	return ret - reallen;
+}
+
+static char *
+recode_credentials(const char *userinfo)
+{
+	char *ui, *creds;
+	size_t ulen;
+
+	/* url-decode the user and pass */
+	ui = url_decode(userinfo);
+
+	ulen = strlen(ui);
+	if (base64_encode(ui, ulen, &creds) == -1)
+		errx(1, "error in base64 encoding");
+	free(ui);
+	return (creds);
+}
+
+/*
+ * Parse a proxy URI and split it up into host, port and userinfo.
+ */
+static void
+proxy_parse_uri(char *uri)
+{
+	char *host, *port = NULL, *cred, *cookie = NULL;
+
+	if (uri == NULL)
+		return;
+
+	if (strncasecmp(uri, "http://", 7) != 0)
+		errx(1, "%s: http_proxy not using http schema", http_info(uri));
+
+	host = uri + 7;
+	if ((host = strndup(host, strcspn(host, "/"))) == NULL)
+		err(1, NULL);
+
+	cred = host;
+	host = strchr(cred, '@');
+	if (host != NULL)
+		*host++ = '\0';
+	else {
+		host = cred;
+		cred = NULL;
+	}
+
+	if (*host == '[') {
+		char *hosttail;
+
+		if ((hosttail = strrchr(host, ']')) == NULL)
+			errx(1, "%s: unmatched opening bracket",
+			     http_info(uri));
+		if (hosttail[1] == '\0' || hosttail[1] == ':')
+			host++;
+		if (hosttail[1] == ':')
+			port = hosttail + 2;
+		*hosttail = '\0';
+	} else {
+		if ((port = strrchr(host, ':')) != NULL)
+			*port++ = '\0';
+	}
+
+	if (port == NULL)
+		port = "443";
+
+	if (cred != NULL) {
+		if (strchr(cred, ':') == NULL)
+			errx(1, "%s: malformed proxy url", http_info(uri));
+		cred = recode_credentials(cred);
+		if (asprintf(&cookie, "Proxy-Authorization: Basic %s\r\n",
+		    cred) == -1)
+			err(1, NULL);
+		free(cred);
+	} else
+		if ((cookie = strdup("")) == NULL)
+			err(1, NULL);
+
+	proxy.proxyhost = host;
+	proxy.proxyport = port;
+	proxy.proxyauth = cookie;
+}
+
 /*
  * Parse a URI and split it up into host, port and path.
  * Does some basic URI validation. Both host and port need to be freed
@@ -301,8 +444,13 @@ http_parse_uri(char *uri, char **ohost, char **oport, char **opath)
 		warnx("%s: preposterous host length", http_info(uri));
 		return -1;
 	}
+	
+	if (memchr(host, '@', path - host) != NULL) {
+		warnx("%s: URI with userinfo not supported", http_info(uri));
+		return -1;
+	}
+
 	if (*host == '[') {
-		char *scope;
 		if ((hosttail = memrchr(host, ']', path - host)) == NULL) {
 			warnx("%s: unmatched opening bracket", http_info(uri));
 			return -1;
@@ -311,8 +459,6 @@ http_parse_uri(char *uri, char **ohost, char **oport, char **opath)
 			host++;
 		if (hosttail[1] == ':')
 			port = hosttail + 2;
-		if ((scope = memchr(host, '%', hosttail - host)) != NULL)
-			hosttail = scope;
 	} else {
 		if ((hosttail = memrchr(host, ':', path - host)) != NULL)
 			port = hosttail + 1;
@@ -514,12 +660,19 @@ http_new(struct http_request *req)
 	LIST_INSERT_HEAD(&active, conn, entry);
 	http_conn_count++;
 
-	/* TODO proxy support (overload of host and port) */
-
-	if (http_resolv(&conn->res0, conn->host, conn->port) == -1) {
-		http_req_fail(req->id);
-		http_free(conn);
-		return;
+	if (proxy.proxyhost != NULL) {
+		if (http_resolv(&conn->res0, proxy.proxyhost,
+		    proxy.proxyport) == -1) {
+			http_req_fail(req->id);
+			http_free(conn);
+			return;
+		}
+	} else {
+		if (http_resolv(&conn->res0, conn->host, conn->port) == -1) {
+			http_req_fail(req->id);
+			http_free(conn);
+			return;
+		}
 	}
 
 	/* connect and start request */
@@ -636,7 +789,7 @@ http_do(struct http_connection *conn, enum res (*f)(struct http_connection *))
 }
 
 /*
- * Connection successfully establish, initiate TLS handshake.
+ * Connection successfully establish, initiate TLS handshake or proxy request.
  */
 static enum res
 http_connect_done(struct http_connection *conn)
@@ -645,12 +798,8 @@ http_connect_done(struct http_connection *conn)
 	conn->res0 = NULL;
 	conn->res = NULL;
 
-#if 0
-	/* TODO proxy connect */
-	if (proxyenv)
-		proxy_connect(conn->fd, sslhost, proxy_credentials); */
-#endif
-
+	if (proxy.proxyhost != NULL)
+		return proxy_connect(conn);
 	return http_tls_connect(conn);
 }
 
@@ -793,10 +942,42 @@ http_tls_handshake(struct http_connection *conn)
 		return WANT_POLLOUT;
 	}
 
-	/* ready to send request */
 	return http_request(conn);
 }
 
+static enum res
+proxy_connect(struct http_connection *conn)
+{
+	char *host;
+	int r;
+
+	assert(conn->state == STATE_CONNECT);
+	conn->state = STATE_PROXY_REQUEST;
+
+	/* Construct the Host header from host and port info */
+	if (strchr(conn->host, ':')) {
+		if (asprintf(&host, "[%s]:%s", conn->host, conn->port) == -1)
+			err(1, NULL);
+
+	} else {
+		if (asprintf(&host, "%s:%s", conn->host, conn->port) == -1)
+			err(1, NULL);
+	}
+
+	free(conn->buf);
+	conn->bufpos = 0;
+	/* XXX handle auth */
+	if ((r = asprintf(&conn->buf, "CONNECT %s HTTP/1.1\r\n"
+	    "User-Agent: " HTTP_USER_AGENT "\r\n%s\r\n", host,
+	    proxy.proxyauth)) == -1)
+		err(1, NULL);
+	conn->bufsz = r;
+
+	free(host);
+
+	return proxy_write(conn);
+}
+
 /*
  * Build the HTTP request and send it out.
  */
@@ -865,18 +1046,25 @@ http_request(struct http_connection *conn)
 
 /*
  * Parse the HTTP status line.
- * Return 0 for status codes 200, 301-304, 307-308.
+ * Return 0 for status codes 100, 103, 200, 203, 301-304, 307-308.
+ * The other 1xx and 2xx status codes are explicitly not handled and are
+ * considered an error.
  * Failure codes and other errors return -1.
  * The redirect loop limit is enforced here.
  */
 static int
 http_parse_status(struct http_connection *conn, char *buf)
 {
+#define HTTP_11	"HTTP/1.1 "
 	const char *errstr;
 	char *cp, ststr[4];
 	char gerror[200];
 	int status;
 
+	/* Check if the protocol is 1.1 and enable keep-alive in that case */
+	if (strncmp(buf, HTTP_11, strlen(HTTP_11)) == 0)
+		conn->keep_alive = 1;
+
 	cp = strchr(buf, ' ');
 	if (cp == NULL) {
 		warnx("Improper response from %s", http_info(conn->host));
@@ -885,7 +1073,7 @@ http_parse_status(struct http_connection *conn, char *buf)
 		cp++;
 
 	strlcpy(ststr, cp, sizeof(ststr));
-	status = strtonum(ststr, 200, 599, &errstr);
+	status = strtonum(ststr, 100, 599, &errstr);
 	if (errstr != NULL) {
 		strnvis(gerror, cp, sizeof gerror, VIS_SAFE);
 		warnx("Error retrieving %s: %s", http_info(conn->host),
@@ -894,19 +1082,23 @@ http_parse_status(struct http_connection *conn, char *buf)
 	}
 
 	switch (status) {
-	case 301:
-	case 302:
-	case 303:
-	case 307:
-	case 308:
+	case 301:	/* Redirect: moved permanently */
+	case 302:	/* Redirect: found / moved temporarily */
+	case 303:	/* Redirect: see other */
+	case 307:	/* Redirect: temporary redirect */
+	case 308:	/* Redirect: permanent redirect */
 		if (conn->req->redirect_loop++ > 10) {
 			warnx("%s: Too many redirections requested",
 			    http_info(conn->host));
 			return -1;
 		}
 		/* FALLTHROUGH */
-	case 200:
-	case 304:
+	case 100:	/* Informational: continue (ignored) */
+	case 103:	/* Informational: early hints (ignored) */
+		/* FALLTHROUGH */
+	case 200:	/* Success: OK */
+	case 203:	/* Success: non-authoritative information (proxy) */
+	case 304:	/* Redirect: not modified */
 		conn->status = status;
 		break;
 	default:
@@ -931,6 +1123,14 @@ http_isredirect(struct http_connection *conn)
 	return 0;
 }
 
+static inline int
+http_isok(struct http_connection *conn)
+{
+	if (conn->status >= 200 && conn->status < 300)
+		return 1;
+	return 0;
+}
+
 static void
 http_redirect(struct http_connection *conn)
 {
@@ -1033,7 +1233,9 @@ http_parse_header(struct http_connection *conn, char *buf)
 	} else if (strncasecmp(cp, CONNECTION, sizeof(CONNECTION) - 1) == 0) {
 		cp += sizeof(CONNECTION) - 1;
 		cp[strcspn(cp, " \t")] = '\0';
-		if (strcasecmp(cp, "keep-alive") == 0)
+		if (strcasecmp(cp, "close") == 0)
+			conn->keep_alive = 0;
+		else if (strcasecmp(cp, "keep-alive") == 0)
 			conn->keep_alive = 1;
 	} else if (strncasecmp(cp, LAST_MODIFIED,
 	    sizeof(LAST_MODIFIED) - 1) == 0) {
@@ -1115,7 +1317,7 @@ read_more:
 	s = tls_read(conn->tls, conn->buf + conn->bufpos,
 	    conn->bufsz - conn->bufpos);
 	if (s == -1) {
-		warn("%s: TLS read: %s", http_info(conn->host),
+		warnx("%s: TLS read: %s", http_info(conn->host),
 		    tls_error(conn->tls));
 		return http_failed(conn);
 	} else if (s == TLS_WANT_POLLIN) {
@@ -1135,10 +1337,40 @@ read_more:
 
 again:
 	switch (conn->state) {
+	case STATE_PROXY_STATUS:
+		buf = http_get_line(conn);
+		if (buf == NULL)
+			goto read_more;
+		if (http_parse_status(conn, buf) == -1) {
+			free(buf);
+			return http_failed(conn);
+		}
+		free(buf);
+		conn->state = STATE_PROXY_RESPONSE;
+		goto again;
+	case STATE_PROXY_RESPONSE:
+		while (1) {
+			buf = http_get_line(conn);
+			if (buf == NULL)
+				goto read_more;
+			/* empty line, end of header */
+			if (*buf == '\0') {
+				free(buf);
+				break;
+			}
+			free(buf);
+		}
+		/* proxy is ready to take connection */
+		if (conn->status == 200) {
+			conn->state = STATE_CONNECT;
+			return http_tls_connect(conn);
+		}
+		return http_failed(conn);
 	case STATE_RESPONSE_STATUS:
 		buf = http_get_line(conn);
 		if (buf == NULL)
 			goto read_more;
+
 		if (http_parse_status(conn, buf) == -1) {
 			free(buf);
 			return http_failed(conn);
@@ -1165,7 +1397,7 @@ again:
 		}
 
 		/* Check status header and decide what to do next */
-		if (conn->status == 200 || http_isredirect(conn)) {
+		if (http_isok(conn) || http_isredirect(conn)) {
 			if (http_isredirect(conn))
 				http_redirect(conn);
 
@@ -1174,6 +1406,8 @@ again:
 			else
 				conn->state = STATE_RESPONSE_DATA;
 			goto again;
+		} else if (conn->status == 100 || conn->status == 103) {
+			conn->state = STATE_RESPONSE_STATUS;
 		} else if (conn->status == 304) {
 			return http_done(conn, HTTP_NOT_MOD);
 		}
@@ -1216,6 +1450,7 @@ again:
 			free(buf);
 			return http_failed(conn);
 		}
+		free(buf);
 
 		/*
 		 * check if transfer is done, in which case the last trailer
@@ -1290,6 +1525,97 @@ http_write(struct http_connection *conn)
 	return http_read(conn);
 }
 
+static enum res
+proxy_read(struct http_connection *conn)
+{
+	ssize_t s;
+	char *buf;
+	int done;
+
+	s = read(conn->fd, conn->buf + conn->bufpos,
+	    conn->bufsz - conn->bufpos);
+	if (s == -1) {
+		warn("%s: read", http_info(conn->host));
+		return http_failed(conn);
+	}
+
+	if (s == 0) {
+		if (conn->req)
+			warnx("%s: short read, connection closed",
+			    http_info(conn->host));
+		return http_failed(conn);
+	}
+
+	conn->bufpos += s;
+
+again:
+	switch (conn->state) {
+	case STATE_PROXY_STATUS:
+		buf = http_get_line(conn);
+		if (buf == NULL)
+			return WANT_POLLIN;
+		if (http_parse_status(conn, buf) == -1) {
+			free(buf);
+			return http_failed(conn);
+		}
+		free(buf);
+		conn->state = STATE_PROXY_RESPONSE;
+		goto again;
+	case STATE_PROXY_RESPONSE:
+		done = 0;
+		while (!done) {
+			buf = http_get_line(conn);
+			if (buf == NULL)
+				return WANT_POLLIN;
+			/* empty line, end of header */
+			if (*buf == '\0')
+				done = 1;
+			free(buf);
+		}
+		/* proxy is ready, connect to remote */
+		if (conn->status == 200) {
+			conn->state = STATE_CONNECT;
+			return http_tls_connect(conn);
+		}
+		return http_failed(conn);
+	default:
+		errx(1, "unexpected http state");
+	}
+}
+
+/*
+ * Send out the proxy request. When done, replace buffer with the read buffer.
+ */
+static enum res
+proxy_write(struct http_connection *conn)
+{
+	ssize_t s;
+
+	assert(conn->state == STATE_PROXY_REQUEST);
+
+	s = write(conn->fd, conn->buf + conn->bufpos,
+		    conn->bufsz - conn->bufpos);
+	if (s == -1) {
+		warn("%s: write", http_info(conn->host));
+		return http_failed(conn);
+	}
+	conn->bufpos += s;
+	if (conn->bufpos < conn->bufsz)
+		return WANT_POLLOUT;
+
+	/* done writing, first thing we need the status */
+	conn->state = STATE_PROXY_STATUS;
+
+	/* free write buffer and allocate the read buffer */
+	free(conn->buf);
+	conn->bufpos = 0;
+	conn->bufsz = HTTP_BUF_SIZE;
+	if ((conn->buf = malloc(conn->bufsz)) == NULL)
+		err(1, NULL);
+
+	return WANT_POLLIN;
+}
+
 /*
  * Properly shutdown the TLS session else move connection into free state.
  */
@@ -1370,6 +1696,8 @@ http_handle(struct http_connection *conn)
 {
 	assert (conn->pfd != NULL && conn->pfd->revents != 0);
 
+	conn->io_time = 0;
+
 	switch (conn->state) {
 	case STATE_CONNECT:
 		return http_finish_connect(conn);
@@ -1377,6 +1705,11 @@ http_handle(struct http_connection *conn)
 		return http_tls_handshake(conn);
 	case STATE_REQUEST:
 		return http_write(conn);
+	case STATE_PROXY_REQUEST:
+		return proxy_write(conn);
+	case STATE_PROXY_STATUS:
+	case STATE_PROXY_RESPONSE:
+		return proxy_read(conn);
 	case STATE_RESPONSE_STATUS:
 	case STATE_RESPONSE_HEADER:
 	case STATE_RESPONSE_DATA:
@@ -1402,6 +1735,8 @@ http_handle(struct http_connection *conn)
 static void
 http_setup(void)
 {
+	char *httpproxy;
+
 	tls_config = tls_config_new();
 	if (tls_config == NULL)
 		errx(1, "tls config failed");
@@ -1416,14 +1751,19 @@ http_setup(void)
 		    tls_config_error(tls_config));
 #endif
 
+#ifdef HAVE_PLEDGE
 	/* load cert file from disk now */
 	tls_ca_mem = tls_load_file(tls_default_ca_cert_file(),
 	    &tls_ca_size, NULL);
 	if (tls_ca_mem == NULL)
 		err(1, "tls_load_file: %s", tls_default_ca_cert_file());
 	tls_config_set_ca_mem(tls_config, tls_ca_mem, tls_ca_size);
+#endif
 
-	/* TODO initalize proxy settings */
+        if ((httpproxy = getenv("http_proxy")) != NULL && *httpproxy == '\0')
+		httpproxy = NULL;
+
+	proxy_parse_uri(httpproxy);
 }
 
 void
@@ -1469,6 +1809,17 @@ proc_http(char *bind_addr, int fd)
 		timeout = INFTIM;
 		now = getmonotime();
 		LIST_FOREACH(conn, &active, entry) {
+			if (conn->io_time == 0)
+				conn->io_time = now + HTTP_IO_TIMEOUT;
+
+			if (conn->io_time <= now)
+				timeout = 0;
+			else {
+				int diff = conn->io_time - now; 
+				diff *= 1000;
+				if (timeout == INFTIM || diff < timeout)
+					timeout = diff;
+			}
 			if (conn->state == STATE_WRITE_DATA)
 				pfds[i].fd = conn->req->outfd;
 			else
@@ -1541,6 +1892,11 @@ proc_http(char *bind_addr, int fd)
 			/* check if event is ready */
 			if (conn->pfd != NULL && conn->pfd->revents != 0)
 				http_do(conn, http_handle);
+			else if (conn->io_time <= now) {
+				warnx("%s: timeout, connection closed",
+				    http_info(conn->host));
+				http_do(conn, http_failed);
+			}
 
 			if (conn->state == STATE_FREE)
 				http_free(conn);
