@@ -1,4 +1,4 @@
-/*	$OpenBSD: roa.c,v 1.22 2021/07/28 12:32:14 job Exp $ */
+/*	$OpenBSD: roa.c,v 1.25 2021/09/09 14:15:49 claudio Exp $ */
 /*
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
  *
@@ -36,6 +36,8 @@ struct	parse {
 	struct roa	 *res; /* results */
 };
 
+static ASN1_OBJECT	*roa_oid;
+
 /*
  * Parse IP address (ROAIPAddress), RFC 6482, section 3.3.
  * Returns zero on failure, non-zero on success.
@@ -90,13 +92,8 @@ roa_parse_addr(const ASN1_OCTET_STRING *os, enum afi afi, struct parse *p)
 			    p->fn, ASN1_tag2str(t->type), t->type);
 			goto out;
 		}
-		maxlength = t->value.integer;
 
-		/*
-		 * It's safe to use ASN1_INTEGER_get() here
-		 * because we're not going to have more than signed 32
-		 * bit maximum of length.
-		 */
+		maxlength = t->value.integer;
 		maxlen = ASN1_INTEGER_get(maxlength);
 		if (maxlen < 0) {
 			warnx("%s: RFC 6482 section 3.2: maxLength: "
@@ -104,7 +101,7 @@ roa_parse_addr(const ASN1_OCTET_STRING *os, enum afi afi, struct parse *p)
 			goto out;
 		}
 		if (addr.prefixlen > maxlen) {
-			warnx("%s: prefixlen (%i) larger than maxLength (%ld)",
+			warnx("%s: prefixlen (%d) larger than maxLength (%ld)",
 			    p->fn, addr.prefixlen, maxlen);
 			goto out;
 		}
@@ -114,10 +111,6 @@ roa_parse_addr(const ASN1_OCTET_STRING *os, enum afi afi, struct parse *p)
 		}
 	}
 
-	p->res->ips = recallocarray(p->res->ips, p->res->ipsz, p->res->ipsz + 1,
-	    sizeof(struct roa_ip));
-	if (p->res->ips == NULL)
-		err(1, NULL);
 	res = &p->res->ips[p->res->ipsz++];
 
 	res->addr = addr;
@@ -186,6 +179,12 @@ roa_parse_ipfam(const ASN1_OCTET_STRING *os, struct parse *p)
 		goto out;
 	}
 
+	/* will be called multiple times so use recallocarray */
+	p->res->ips = recallocarray(p->res->ips, p->res->ipsz,
+	    p->res->ipsz + sk_ASN1_TYPE_num(sseq), sizeof(struct roa_ip));
+	if (p->res->ips == NULL)
+		err(1, NULL);
+
 	for (i = 0; i < sk_ASN1_TYPE_num(sseq); i++) {
 		t = sk_ASN1_TYPE_value(sseq, i);
 		if (t->type != V_ASN1_SEQUENCE) {
@@ -342,9 +341,14 @@ roa_parse(X509 **x509, const char *fn)
 	p.fn = fn;
 
 	/* OID from section 2, RFC 6482. */
+	if (roa_oid == NULL) {
+		roa_oid = OBJ_txt2obj("1.2.840.113549.1.9.16.1.24", 1);
+		if (roa_oid == NULL)
+			errx(1, "OBJ_txt2obj for %s failed",
+			    "1.2.840.113549.1.9.16.1.24");
+	}
 
-	cms = cms_parse_validate(x509, fn,
-	    "1.2.840.113549.1.9.16.1.24", &cmsz);
+	cms = cms_parse_validate(x509, fn, roa_oid, &cmsz);
 	if (cms == NULL)
 		return NULL;
 
