@@ -365,11 +365,16 @@ notification_done(struct notification_xml *nxml, char *last_mod)
 	if (nxml->repository->serial == 0)
 		goto snapshot;
 
-	if (nxml->repository->serial == nxml->serial) {
-		nxml->current->serial = nxml->serial;
+	/* if our serial is equal or bigger, the repo is up to date */
+	if (nxml->repository->serial >= nxml->serial) {
+		nxml->current->serial = nxml->repository->serial;
 		return NOTIFICATION;
 	}
 
+	/* it makes no sense to process too many deltas */
+	if (nxml->serial - nxml->repository->serial > 300)
+		goto snapshot;
+
 	/* check that all needed deltas are available */
 	s = nxml->repository->serial + 1;
 	TAILQ_FOREACH(d, &nxml->delta_q, q) {
